from .exceptions import *
from .pyromod import *
from .rpc_error import UnknownError


class BadMsgNotification(Exception):
    descriptions = {
        16: "The msg_id is too low, the client time has to be synchronized.",
        17: "The msg_id is too high, the client time has to be synchronized.",
        18: "Incorrect two lower order of the msg_id bits, the server expects the client message "
        "msg_id to be divisible by 4.",
        19: "The container msg_id is the same as the msg_id of a previously received message.",
        20: "The message is too old, it cannot be verified by the server.",
        32: "The msg_seqno is too low.",
        33: "The msg_seqno is too high.",
        34: "An even msg_seqno was expected, but an odd one was received.",
        35: "An odd msg_seqno was expected, but an even one was received.",
        48: "Incorrect server salt.",
        64: "Invalid container.",
    }

    def __init__(self, code):
        description = self.descriptions.get(code, "Unknown error code")
        super().__init__(f"[{code}] {description}")


class SecurityError(Exception):

    @classmethod
    def check(cls, cond: bool, msg: str):
        if not cond:
            raise cls(f"Check failed: {msg}")


class SecurityCheckMismatch(SecurityError):

    def __init__(self, msg: str = None):
        super().__init__(
            "A security check mismatch has occurred." if msg is None else msg
        )


class CDNFileHashMismatch(SecurityError):

    def __init__(self, msg: str = None):
        super().__init__(
            "A CDN file hash mismatch has occurred." if msg is None else msg
        )
